==> em_tasks-0.1.66-py2.py3-none-any.whl: em_tasks/tools/spapi_products_collector.py <==
import re

from em_tasks.spapi.spapi_catalog_items_parser import SpapiCatalogItemsParser
from em_tasks import logger


class SpapiProductsCollector():
  def __init__(self, keywords, spapi, marketplace, locale='en_US'):
    self.keywords = keywords
    self.spapi = spapi
    self.marketplace = marketplace
    self.locale = locale
    self.search_type = 'keywords'
    self.tasks = []
    self.finished_tasks = {}

    self.cat_keywords = [
      'book', 'tv', 'video', 'food', 'grocery', 'wine',
      'electronic', 'cell phone', 'computer', 'headphone', 'camera', 'photo', 'audio',
      'musical', 'games', 'software', 'renewed', 'cloth', 'toy'
    ]
    self.cat_filter = re.compile(r'\b(' + '|'.join(re.escape(keyword) for keyword in self.cat_keywords) + r')\b')

  def run(self):
    d = {}
    for keyword in self.keywords:
      origin_task = {
        'asins': [keyword],
        'marketplace': self.marketplace,
        'locale': self.locale,
        'search_type': self.search_type
      }
      self.add_task(origin_task)

      while self.has_more_task():
        task = self.get_next_task()

        resp = None
        max_retries = 7
        while max_retries > 0:
          try:
            resp = self.spapi.search_catalog_items(**task)
            if resp:
              break
          except:
            max_retries -= 1

        if not resp:
          continue

        # Process items
        items = SpapiCatalogItemsParser.parse(resp)
        yield items

        item_search_results = resp.payload

        pagination = item_search_results.get('pagination', None) or d
        next_token = pagination.get('nextToken', None)
        if next_token:
          next_page_task = dict(task)
          next_page_task['pageToken'] = next_token
          if not self.is_task_added(next_page_task):
            self.add_task(next_page_task)

          continue

        self.add_to_finished_tasks(task)

        # Process refinements
        refinements = item_search_results.get('refinements', None) or d

        classifications = refinements.get('classifications', None) or d
        if classifications:
          for classification in classifications:
            passed = self.filter_category(classification)
            if not passed:
              continue

            cat_task = dict(task)
            cat_task.pop('pageToken', None)

            cat_task['classificationIds'] = str(classification['classificationId'])

            if not self.is_task_added(cat_task) and not self.is_task_finished(cat_task):
              self.add_task(cat_task)

        # Ignore brands now
        brands = refinements.get('brands', None) or d

  def has_more_task(self):
    return bool(self.tasks)

  def get_next_task(self):
    return self.tasks.pop(0)

  def add_task(self, task):
    self.tasks.append(task)
    logger.debug('[AddedTask] {}'.format(task))

  def is_task_added(self, task):
    same = False
    for t in self.tasks:
      same = t['asins'] == task['asins'] and t.get('pageToken', None) == task.get('pageToken', None) and t.get('classificationIds', None) == task.get('classificationIds', None)
      if same:
        break

    return same

  def is_task_finished(self, task):
    k = self.get_task_key(task)
    return k in self.finished_tasks

  def add_to_finished_tasks(self, task):
    k = self.get_task_key(task)
    self.finished_tasks[k] = None

  def get_task_key(self, task):
    keyword = '-'.join(task['asins'])
    classification_ids = task.get('classificationIds', None)
    return '{}-{}'.format(keyword, classification_ids)

  def filter_category(self, classification):
    classification_id = classification.get('classificationId', None)
    if not classification_id:
      return False

    results_cnt = classification.get('numberOfResults', 0)
    if results_cnt <= 0:
      return False

    cat_name = classification.get('displayName', None)
    if not cat_name:
      return False

    filtered = not bool(self.cat_filter.search(cat_name, re.IGNORECASE))
    return filtered

==> em_tasks-0.1.69-py2.py3-none-any.whl: em_tasks/tools/spapi_products_collector.py <==
import re

from em_tasks.spapi.spapi_catalog_items_parser import SpapiCatalogItemsParser
from em_tasks import logger


class SpapiProductsCollector():
  def __init__(self, keywords, spapi, marketplace, locale='en_US'):
    self.keywords = keywords
    self.spapi = spapi
    self.marketplace = marketplace
    self.locale = locale
    self.search_type = 'keywords'
    self.tasks = []
    self.finished_tasks = {}

    self.cat_keywords = [
      'book', 'tv', 'video', 'food', 'grocery', 'wine',
      'electronic', 'cell phone', 'computer', 'headphone', 'camera', 'photo', 'audio',
      'musical', 'games', 'software', 'renewed', 'cloth', 'toy'
    ]
    self.cat_filter = re.compile(r'\b(' + '|'.join(re.escape(keyword) for keyword in self.cat_keywords) + r')\b')

  def run(self):
    d = {}
    for keyword in self.keywords:
      origin_task = {
        'asins': [keyword],
        'marketplace': self.marketplace,
        'locale': self.locale,
        'search_type': self.search_type
      }
      self.add_task(origin_task)

      while self.has_more_task():
        task = self.get_next_task()

        resp = None
        max_retries = 7
        while max_retries > 0:
          try:
            resp = self.spapi.search_catalog_items(**task)
            if resp:
              break
          except:
            max_retries -= 1

        if not resp:
          continue

        # Process items
        items = SpapiCatalogItemsParser.parse(resp)
        yield items

        item_search_results = resp.payload

        pagination = item_search_results.get('pagination', None) or d
        next_token = pagination.get('nextToken', None)
        if next_token:
          next_page_task = dict(task)
          next_page_task['pageToken'] = next_token
          if not self.is_task_added(next_page_task):
            self.add_task(next_page_task)

          continue

        self.add_to_finished_tasks(task)

        # Process refinements
        refinements = item_search_results.get('refinements', None) or d

        classifications = refinements.get('classifications', None) or d
        if classifications:
          for classification in classifications:
            passed = self.filter_category(classification)
            if not passed:
              continue

            cat_task = dict(task)
            cat_task.pop('pageToken', None)

            cat_task['classificationIds'] = str(classification['classificationId'])

            if not self.is_task_added(cat_task) and not self.is_task_finished(cat_task):
              self.add_task(cat_task)

        # Ignore brands now
        brands = refinements.get('brands', None) or d
        if brands:
          for brand in brands:
            brand_name = brand.get('brandName', None)
            if not brand_name:
              continue

            brand_task = dict(task)
            brand_task.pop('pageToken', None)

            brand_task['brandNames'] = str(brand_name)

            if not self.is_task_added(brand_task) and not self.is_task_finished(brand_task):
              self.add_task(brand_task)

  def has_more_task(self):
    return bool(self.tasks)

  def get_next_task(self):
    return self.tasks.pop(0)

  def add_task(self, task):
    self.tasks.append(task)
    logger.debug('[AddedTask] {}'.format(task))

  def is_task_added(self, task):
    same = False
    for t in self.tasks:
      is_keywords_same = t['asins'] == task['asins']
      is_page_same = t.get('pageToken', None) == task.get('pageToken', None)
      is_cat_same = t.get('classificationIds', None) == task.get('classificationIds', None)
      is_brand_same = t.get('brandNames', None) == task.get('brandNames', None)
      same = is_keywords_same and is_page_same and is_cat_same and is_brand_same
      if same:
        break

    return same

  def is_task_finished(self, task):
    k = self.get_task_key(task)
    return k in self.finished_tasks

  def add_to_finished_tasks(self, task):
    k = self.get_task_key(task)
    self.finished_tasks[k] = None

  def get_task_key(self, task):
    keyword = '-'.join(task['asins'])
    classification_ids = task.get('classificationIds', None)
    brand_names = task.get('brandNames', None)
    return '{}-{}-{}'.format(keyword, classification_ids, brand_names)

  def filter_category(self, classification):
    classification_id = classification.get('classificationId', None)
    if not classification_id:
      return False

    results_cnt = classification.get('numberOfResults', 0)
    if results_cnt <= 0:
      return False

    cat_name = classification.get('displayName', None)
    if not cat_name:
      return False

    filtered = not bool(self.cat_filter.search(cat_name, re.IGNORECASE))
    return filtered
